# Copyright (c) 2011, Intel Corporation
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright notice,
#       this list of conditions and the following disclaimer in the documentation
#       and/or other materials provided with the distribution.
#     * Neither the name of Intel Corporation nor the names of its contributors
#       may be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""bits module."""

import _bits
from _bits import *

_grub_command_map = {}

def register_grub_command(command, func, summary, description):
    """Register a new GRUB command, implemented using the given callable.
    The callable should accept a single argument, the list of argument strings.
    The arguments include the command name as [0]."""
    _bits._register_grub_command(command, summary, description)
    _grub_command_map[command] = func

def _grub_command_callback(args):
    try:
        return _grub_command_map[args[0]](args)
    except:
        import traceback
        traceback.print_exc()
        return False

_bits._set_grub_command_callback(_grub_command_callback)

def bsp_apicid():
    """Returns the BSP's APIC ID."""
    cpulist = cpus()
    return cpulist[0]

def socket_index(apic_id):
    """Returns the socket portion of the APIC ID"""
    if cpuid(apic_id,0x0)[0] < 0xb:
        return None
    eax, ebx, ecx, edx = cpuid(apic_id,0xb,1)
    x2apic_id = edx
    socket_index = x2apic_id >> (eax & 0x1f);
    return socket_index

def socket_apic_ids():
    """Returns a mapping to unique socket index with the list of APIC IDs"""
    uniques = {}
    cpulist = cpus()
    for apicid in cpulist:
        uniques.setdefault(socket_index(apicid), []).append(apicid)
    return uniques

def apicid_to_index():
    """Returns a reverse mapping from APIC ID to CPU number"""
    cpulist = bits.cpus()
    return dict([(apicid, i) for (i, apicid) in enumerate(cpulist)])
