/*
Copyright (c) 2011, Intel Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.
    * Neither the name of Intel Corporation nor the names of its contributors
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "Python.h"
#include "pyunconfig.h"

#include <grub/types.h>
#include <grub/dl.h>
#include <grub/env.h>
#include <grub/err.h>
#include <grub/extcmd.h>
#include <grub/misc.h>

static const struct grub_arg_option py_options_options[] = {
#undef OPTION_VERBOSE
#define OPTION_VERBOSE 0
    {"verbose", 'v', 0, "Set the Verbose level (default=0)\n"
     "    0 = No verbose details on module initialization or exit.\n"
     "    1 = Print a message each time a module is initialized, showing \n"
     "        the place (filename or built-in module) from which it is loaded.\n"
     "    2 = Print a message for each file that is checked for when searching\n"
     "        for a module. Also provides information on module cleanup at exit.",
     "NUM", ARG_TYPE_INT},
    {0, 0, 0, 0, 0, 0}
};

static grub_err_t grub_cmd_py_options(struct grub_extcmd *cmd, int argc, char **args)
{
    (void)argc;
    (void)args;

    if (cmd->state[OPTION_VERBOSE].set)
        Py_VerboseFlag = grub_strtoul(cmd->state[OPTION_VERBOSE].arg, NULL, 0);
    else
        grub_printf("Py_VerboseFlag = %u\n", Py_VerboseFlag);

    return GRUB_ERR_NONE;
}

static grub_err_t grub_cmd_py(grub_command_t cmd, int argc, char **args)
{
    (void)cmd;
    if (argc == 1)
        PyRun_SimpleString(args[0]);
    return GRUB_ERR_NONE;
}

static grub_command_t cmd_py;
static grub_extcmd_t cmd_py_options;

GRUB_MOD_INIT(python)
{
    __asm__ __volatile__ ("finit");
    Py_DontWriteBytecodeFlag = 1;
    Py_NoSiteFlag = 1;
    Py_Initialize();
    cmd_py = grub_register_command("py", grub_cmd_py, "\"Python program\"", "Evaluate Python given on the command line.");
    cmd_py_options = grub_register_extcmd("py_options", grub_cmd_py_options, GRUB_COMMAND_FLAG_BOTH,
                                          "[-v NUM]",
                                          "Set python options",
                                          py_options_options);
}

GRUB_MOD_FINI(python)
{
    grub_unregister_command(cmd_py);
    grub_unregister_extcmd(cmd_py_options);
    Py_Finalize();
}
